//////////////////////////////////////////////////////////////
/////           Divers schemas de volumes finis        ///////
/////               pour pbs elliptiques 1D            ///////
/////                                                  ///////
/////              différents schémas de VF            ///////
/////                                                  ///////
/////                F. Boyer  &  S. Krell             ///////
/////                     Mars 2010                    ///////
//////////////////////////////////////////////////////////////

// Toutes ces fonctions prennent deux paramètres :
//      m        : un maillage
//      donnees  : un jeu de donnees
//

//**************************************************************************
//    Le schéma de base pour le Laplacien 
//    avec conditions de Dirichlet homogène
//**************************************************************************
function [A,b]=vf_laplacien_dirh(m,donnees);

  printf('Assemblage du schéma VF - Laplacien Dirichlet homogène - %g mailles\n',m.nb_vol);
    
  A=spzeros(m.nb_vol,m.nb_vol);
  b=zeros(m.nb_vol,1);
  
  for i=2:m.nb_vol-1
    A(i,i)= (1/m.dist(i)+1/m.dist(i+1));
    A(i,i-1)=- 1/m.dist(i);
    A(i,i+1)=- 1/m.dist(i+1);
  end  
  
  A(1,1)=(1/m.dist(1)+1/m.dist(2));
  A(1,2)=-1/m.dist(2);
  A(m.nb_vol,m.nb_vol)= (1/m.dist(m.nb_vol+1)+1/m.dist(m.nb_vol));
  A(m.nb_vol,m.nb_vol-1)= - 1/m.dist(m.nb_vol);

  b=feval(m.centres,donnees.source).*m.mes;

endfunction

//**************************************************************************
//    Le schéma de base pour le Laplacien 
//    avec conditions de Dirichlet non homogène
//**************************************************************************
function [A,b]=vf_laplacien_dirnh(m,donnees);

  printf('Assemblage du schéma VF - Laplacien Dirichlet non homogène - %g mailles\n',m.nb_vol);
  
  printf('La fonction ''vf_laplacien_dirnh'' n''est pas encore programmee !\n');
  abort


   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
     
  
endfunction

//**************************************************************************
//    Routine de calcul du coefficient de diffusion aux interfaces
//        selon la méthode choisie
//**************************************************************************
function [ck]=calcul_coeff_k_interf(m,donnees);
  
  if ~isfield(donnees,"methode") then
    donnees.methode="exacte";
  end;
  
  printf('La fonction ''calcul_coeff_k_interf'' n''est pas encore programmee !\n');
  abort
  

   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
     
  
  
endfunction

//**************************************************************************
//    Le schéma général pour le problème de diffusion
//    avec conditions de Dirichlet non homogène
//       - prise en compte des divers calculs du coefficient de diffusion
//             aux interfaces
//       - prise en compte d'un éventuel terme de Dirac
//**************************************************************************
function [A,b]=vf_diffusion_dirnh(m,donnees);

  printf('Assemblage du schéma VF - Diffusion Dirichlet non homogène - %g mailles\n',m.nb_vol);

  printf('La fonction ''vf_diffusion_dirnh'' n''est pas encore programmee !\n');
  abort

   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
       
endfunction
  
//**************************************************************************
//    Le schéma général pour le problème de diffusion
//    avec conditions de Neumann non homogène
//       - prise en compte des divers calculs du coefficient de diffusion
//             aux interfaces
//       - prise en compte d'un éventuel terme de Dirac
//**************************************************************************
  
function [A,b]=vf_diffusion_neunh(m,donnees);
 
  printf('Assemblage du schéma VF - Diffusion - Neumann non homogène - %g mailles\n',m.nb_vol);
  
  if ~isfield(donnees,"bordN") then
     printf("Pas de donnee au bord de type Neumann !");
     abort;
  end;

  if ~isfield(donnees,"moyenne") then
     printf("Pour un calcul de Neumann il faut fixer la moyenne !\n");
     abort;
  end;

  printf('La fonction ''vf_diffusion_neunh'' n''est pas encore programmee !\n');
  abort

   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
       
endfunction

//**************************************************************************
//    Le schéma général pour le problème de diffusion
//    avec conditions de Neumann non homogène
//       - prise en compte des divers calculs du coefficient de diffusion
//             aux interfaces
//       - prise en compte d'un éventuel terme de Dirac
//**************************************************************************
  
function [A,b]=vf_diffusion_fournh(m,donnees);
 
  printf('Assemblage du schéma VF - Diffusion - Fourier non homogène - %g mailles\n',m.nb_vol);
  
  if ~isfield(donnees,"bordF") then
     printf("Pas de donnee au bord de type Fourier !");
     abort;
  end;
  
  if ~isfield(donnees,"coeffF") then
     printf("Pas de coefficient pour les CL de Fourier !");
     abort;
  end;
  

  printf('La fonction ''vf_diffusion_fournh'' n''est pas encore programmee !\n');
  abort

   //////////////////////////////////////////////////////////////////////
   //     #              #######     #      #####   ######   ####### 
   //     #              #           #        #     #     #  #       
   //    ###             #          ###       #     #     #  #       
   //    # #             #####      # #       #     ######   #####   
   //   #####            #         #####      #     #   #    #       
   //   #   #            #         #   #      #     #    #   #       
   //  ##   ##           #        ##   ##   #####   #     #  ####### 
   //////////////////////////////////////////////////////////////////////
       
endfunction